#ifndef MD3IMPORT_H
#define MD3IMPORT_H

#include "md3tomdl.h"

//for the q3 data, structs are fixed size objects which exactly match the size of the data in the flie
//this way they can be read as a lump with a char cast straight from the file
//anything which requires heap allocation is a class instead, has vectors to handle the variable size data
//these are also allowed to have additional helper fields

struct q3_header 
{
	int8_t ident[4];
	int32_t version;
	char name[64];
	int32_t flags;
	int32_t framecount;
	int32_t tagcount;
	int32_t surfacecount;
	int32_t skincount;
	int32_t ofs_frames;
	int32_t ofs_tags;
	int32_t ofs_surfaces;
	int32_t ofs_eof;
};

struct q3_vertex 
{
	int16_t x;
	int16_t y;
	int16_t z;
	int16_t norm;
};


struct q3_triangle 
{
	int32_t index[3];
};

struct q3_frame 
{
	float mins[3];
	float maxs[3];
	float org[3];
	float radius;
	int8_t name[16];
};

class q3_frame_vertices
{
public:
	std::vector <q3_vertex> vertex;
};



struct q3_texcoord 
{
	float u;
	float v;
};


struct q3_surfaceheader
{
	int8_t ident[4];
	int8_t name[64];
	int32_t flags;

	int32_t num_frames;
	int32_t num_shaders;
	int32_t num_verts;
	int32_t num_triangles;

	int32_t ofs_triangles;
	int32_t ofs_shaders;
	int32_t ofs_st;
	int32_t ofs_vertices;
	int32_t ofs_end;
};

class q3_surface 
{
public:
	q3_surfaceheader header;
	int filepoint;	//records the offset into the file of the top of the header
	std::vector <q3_triangle> triangle;
	std::vector <q3_texcoord> texcoord;
	std::vector <q3_frame_vertices> frame; //note an extra layer of object here
};


class q3_model
{
public:
	q3_header header;
	std::vector <q3_frame> frames;
	std::vector <q3_surface> surfaces;
};

static float MD3_XYZ_SCALE = 1.0f / 64;
int32_t parse_q3model(std::ifstream &q3input, q3_model &q3mdl);
#endif